@@ -7,10 +7,9 @@ use warnings;
 
 use Carp qw( carp croak );
 use Path::Tiny;
-use Symbol ();
 
 use vars qw( $VERSION );
-$VERSION = '2.05'; 
+$VERSION = '2.06'; 
 
 use Object::Tiny::RW 
 	'comment',			# True to create the %%[] comment macro
@@ -36,7 +35,6 @@ use Object::Tiny::RW
 
 sub _expand_variable { # Private object method.
     my $self  = shift;
-    my $class = ref( $self ) || $self;
     local $_  = (shift || '');
 
     foreach my $var (  
@@ -57,7 +55,6 @@ sub _expand_variable { # Private object method.
 # Algorithms with Perl (with the errata).
 sub _find_element { # Private object method.
     my( $self, $array_name, $target ) = @_;
-    my $class = ref( $self ) || $self;
 
     my $target_len    = length $target;
 
@@ -92,7 +89,6 @@ sub _find_element { # Private object method.
 
 sub _insert_element { # Private object method.
     my( $self, $array_name, $name, $body ) = @_;
-    my $class = ref( $self ) || $self;
 
     if( $array_name eq 'VARIABLE' ) {
         $self->{$array_name}{$name} = $body;
@@ -115,7 +111,6 @@ sub _insert_element { # Private object method.
 
 sub _remove_element { # Private object method.
     my( $self, $array_name, $name ) = @_;
-    my $class = ref( $self ) || $self;
 
     my $element = undef;
 
@@ -200,7 +195,6 @@ sub new { # Class and object method
 # deprecated method to define -macro, -script or -variable
 sub define { # Object method.
     my( $self, $which, $name, $body ) = @_;
-    my $class = ref( $self ) || $self;
 
     $self->_insert_element( uc substr( $which, 1 ), $name, $body );
 }
@@ -209,11 +203,9 @@ sub define { # Object method.
 # deprecated method to undefine -macro, -script or -variable
 sub undefine { # Object method.
     my( $self, $which, $name ) = @_;
-    my $class = ref( $self ) || $self;
 
     $which = uc substr( $which, 1 );
-
-    carp "No $which called $name exists" unless 
+    carp "Cannot undefine non-existent $which $name" unless 
     $self->_remove_element( $which, $name ); 
 }
 
@@ -222,7 +214,6 @@ sub undefine { # Object method.
 # deprecated method to list all -macro, -script or -variable
 sub list { # Object method.
     my( $self, $which, $namesonly ) = @_;
-    my $class = ref( $self ) || $self;
 
     my @lines;
     local $_;
@@ -268,7 +259,6 @@ sub list { # Object method.
 # deprecated method to undefine all -macro, -script or -variable
 sub undefine_all { # Object method.
     my( $self, $which ) = @_;
-    my $class = ref( $self ) || $self;
 
     $which = uc substr( $which, 1 );
 
@@ -405,7 +395,6 @@ sub _define_standard_comment {
 # load macro definitions from a file
 sub load_file { # Object method.
     my( $self, $file ) = @_;
-    my $class = ref( $self ) || $self;
 
     # Treat loaded files as if wrapped in delimiters (only affects embedded
     # processing).
@@ -425,63 +414,58 @@ sub load_file { # Object method.
 # @expanded = $macro->expand_file( name, body );
 # In a void context will print to the current filehandle
 sub expand_file { # Object method.
-    my( $self, $file, $noprint ) = @_;
-    my $class = ref( $self ) || $self;
+    my($self, $file, $noprint) = @_;
 
     # We need this because eval creates a scalar context.
     my $wantarray = wantarray;
 
     my @lines;
 
-    eval {
-        croak "missing filename"            unless     $file; 
-        croak "file `$file' does not exist" unless  -e $file;
+	croak "Missing filename"            unless     $file; 
+	
+	substr( $file, 0, 1 ) = ( $ENV{HOME} or $ENV{LOGDIR} or (getpwuid( $> ))[7] ) 
+	if substr( $file, 0, 1 ) eq '~';
+	
+	croak "File '$file' does not exist" unless  -e $file;
+    
 
-        substr( $file, 0, 1 ) = ( $ENV{HOME} or $ENV{LOGDIR} or (getpwuid( $> ))[7] ) 
-        if substr( $file, 0, 1 ) eq '~';
+	local $_;
 
-        local $_;
+	open my $fh, $file or croak "Open '$file' failed: $!";
+	my $line_nr;
+	
+	while( <$fh> ) {
+		$line_nr++;
+		my $line = $self->embedded ? 
+						$self->expand_embedded( $_, $file, $line_nr ) :
+						$self->expand( $_, $file, $line_nr );
 
-        my $fh = Symbol::gensym;
+		next unless defined($line) && $line ne ''; 
 
-        open $fh, $file or croak "failed to open $file: $!";
-
-        while( <$fh> ) {
-            my $line = $self->embedded ? 
-                            $self->expand_embedded( $_, $file ) :
-                            $self->expand( $_, $file );
-
-            next unless defined $line and $line; 
-
-            if( $wantarray ) {
-                push @lines, $line;
-            }
-            else {
-                print $line unless $noprint;
-            }
-        }
-
-        close $fh or croak "failed to close $file: $!";
-
-	if( $self->in_macro || $self->in_script ) {
-	    my $which = $self->in_macro ? 'DEFINE' : 'DEFINE_SCRIPT';
-	    croak "runaway \%$which to end of file"
+		if( $wantarray ) {
+			push @lines, $line;
+		}
+		else {
+			print $line unless $noprint;
+		}
 	}
 
-    };
-    croak $@ if $@;
+	close $fh or croak "Close '$file' failed: $!";
 
-    @lines if $wantarray and not $noprint;
+	if( $self->in_macro || $self->in_script ) {
+		my $which = $self->in_macro ? 'DEFINE' : 'DEFINE_SCRIPT';
+		croak "Runaway \%$which from $file line ".$self->line_nr." to end of file"
+	}
+
+    @lines if $wantarray && ! $noprint;
 }
 
 
 #------------------------------------------------------------------------------
 # similar to expand(), but only expands text between the open and close delimiters
 sub expand_embedded { # Object method.
-    my $self  = shift;
-    my $class = ref( $self ) || $self;
-    local $_  = shift;
-    my $file  = (shift || '-');
+	my($self, $text, $file, $line_nr) = @_;
+    local $_ = $text;
 
     my $line = '';
 
@@ -499,12 +483,14 @@ sub expand_embedded { # Object method.
             my $end   = index( $_, $self->closedelim, $start );
             if( $end > -1 ) {
                 $line .= $self->expand( 
-                    substr( $_, $start, $end - $start ), $file );
+									substr( $_, $start, $end - $start ), 
+									$file, $line_nr );
                 $line .= $self->expand_embedded( 
-                            substr( $_, $end + length($self->closedelim) ), $file );
+									substr( $_, $end + length($self->closedelim) ), 
+									$file, $line_nr );
             }
             else {
-                $line .= $self->expand( substr( $_, $start ), $file );
+                $line .= $self->expand( substr( $_, $start ), $file, $line_nr );
                 $self->in_embedded(1);
             }
         }
@@ -515,13 +501,16 @@ sub expand_embedded { # Object method.
     else {
         my $end = index( $_, $self->closedelim );
         if( $end > -1 ) {
-            $line = $self->expand( substr( $_, 0, $end ), $file );
+            $line = $self->expand( 
+								substr( $_, 0, $end ), 
+								$file, $line_nr );
             $self->in_embedded(0);
             $line .= $self->expand_embedded( 
-                        substr( $_, $end + length($self->closedelim) ), $file );
+								substr( $_, $end + length($self->closedelim) ), 
+								$file, $line_nr );
         }
         else {
-            $line = $self->expand( $_, $file );
+            $line = $self->expand( $_, $file, $line_nr );
         }
     }
 
@@ -530,275 +519,271 @@ sub expand_embedded { # Object method.
 
 
 #------------------------------------------------------------------------------
-# parse and expand passed string; filename is used for error messages
+# parse and expand passed string; file and line_nr are used for error messages
 sub expand { # Object method.
-    my $self  = shift;
-    my $class = ref( $self ) || $self;
-    local $_  = shift;
-    my $file  = (shift || '-');
+	my($self, $text, $file, $line_nr) = @_;
+    local $_ = $text;
+	
+    $file //= '-';
+	$line_nr ||= 1;
+    $self->line_nr($line_nr) unless ($self->in_macro || $self->in_script);
+    my $where = "at $file line ".$self->line_nr;
+	my $where_to = "from $file line ".$self->line_nr." to line $line_nr";
+	
+	if( /^\%((?:END_)?CASE)(?:\s*\[(.*?)\])?/mso || 
+		( ($self->in_case || '') eq 'SKIP' ) ) {
 
-    $self->line_nr($. || 1) unless ($self->in_macro || $self->in_script);
-    my $where     = "at $file line ".$self->line_nr;
+		croak "Runaway \%DEFINE $where_to" if $self->in_macro;
+		croak "Runaway \%DEFINE_SCRIPT $where_to" if $self->in_script;
 
-    eval {
-        if( /^\%((?:END_)?CASE)(?:\s*\[(.*?)\])?/mso || 
-            ( ($self->in_case || '') eq 'SKIP' ) ) {
+		if( defined $1 and $1 eq 'CASE' ) {
+			croak "Missing \%CASE condition $where" unless defined $2;
 
-            croak "runaway \%DEFINE $where to line $."
-            if $self->in_macro;
-            croak "runaway \%DEFINE_SCRIPT $where to line $."
-            if $self->in_script;
+			my $eval    = $self->_expand_variable( $2 );
+			my $result;
+			eval {
+				my %Var = %{$self->VARIABLE};
+				local $_;
+				$result = eval $eval;
+				%{$self->VARIABLE} = %Var;
+			};
+			croak "Evaluation of %CASE [$eval] failed $where: $@" if $@;
 
-            if( defined $1 and $1 eq 'CASE' ) {
-                croak "no condition for CASE $where" unless defined $2;
+			$self->in_case($result ? 1 : 'SKIP');
+		}
+		elsif( defined $1 and $1 eq 'END_CASE' ) {
+			$self->in_case(0);
+		}
 
-                my $eval    = $self->_expand_variable( $2 );
-                my $result;
-                eval {
-                    my %Var = %{$self->VARIABLE};
-                    local $_;
-                    $result = eval $eval;
-                    %{$self->VARIABLE} = %Var;
-                };
-                croak "evaluation of CASE $eval failed $where: $@" if $@;
+		$_ = '';
+	}
+	elsif( ( $self->in_macro || $self->in_script ) && /^\%END_DEFINE/mso ) {
+		# End of a multi-line macro or script
+		$self->cur_define( $self->_expand_variable( $self->cur_define ) );
 
-                $self->in_case($result ? 1 : 'SKIP');
-            }
-            elsif( defined $1 and $1 eq 'END_CASE' ) {
-                $self->in_case(0);
-            }
+		if ($self->in_macro) {
+			$self->in_macro(0);
+			$self->_insert_element('MACRO', $self->cur_name, $self->cur_define );
+		}
+		else {
+			$self->in_script(0);
+			$self->_insert_element('SCRIPT', $self->cur_name, $self->cur_define );
+		}
 
-            $_ = '';
-        }
-        elsif( ( $self->in_macro || $self->in_script ) && /^\%END_DEFINE/mso ) {
-            # End of a multi-line macro or script
-            $self->cur_define( $self->_expand_variable( $self->cur_define ) );
+		$self->cur_name('');
+		$self->cur_define('');
+		
+		$_ = '';
+	}
+	elsif( $self->in_macro || $self->in_script ) {
+		# Accumulating the body of a multi-line macro or script
+		my $which = $self->in_macro ? 'DEFINE' : 'DEFINE_SCRIPT';
+		croak "Runaway \%$which $where_to"
+		if /^\%
+			(?:(?:UNDEFINE(?:_ALL)?|DEFINE)(?:_SCRIPT|_VARIABLE)?) |
+			LOAD | INCLUDE | (?:END_)CASE
+		   /msox;
 
-			if ($self->in_macro) {
-				$self->in_macro(0);
-				$self->_insert_element('MACRO', $self->cur_name, $self->cur_define );
-			}
-			else {
-				$self->in_script(0);
-				$self->_insert_element('SCRIPT', $self->cur_name, $self->cur_define );
-			}
+		$self->{cur_define} .= $_;
 
-            $self->cur_name('');
-			$self->cur_define('');
-			
-            $_ = '';
-        }
-        elsif( $self->in_macro || $self->in_script ) {
-            # Accumulating the body of a multi-line macro or script
-            my $which = $self->in_macro ? 'DEFINE' : 'DEFINE_SCRIPT';
-            croak "runaway \%$which $where to line $."
-            if /^\%
-                (?:(?:UNDEFINE(?:_ALL)|DEFINE)(?:_SCRIPT|_VARIABLE)?) |
-                LOAD | INCLUDE | (?:END_)CASE
-               /msox;
+		$_ = '';
+	}
+	elsif( /^\%UNDEFINE(?:_(SCRIPT|VARIABLE))?\s+([^][\s]+)/mso ) {
+		# Undefining a macro, script or variable
+		my $which = $1 || 'MACRO';
 
-            $self->{cur_define} .= $_;
+		carp "Cannot undefine non-existent $which $2 $where" 
+		unless $self->_remove_element( $which, $2 ); 
+ 
+		$_ = '';
+	}
+	elsif( /^\%UNDEFINE_ALL(?:_(SCRIPT|VARIABLE))?/mso ) {
+		# Undefining all macros or scripts
+		my $which = "-".lc($1 || 'MACRO');
+		$self->undefine_all($which);
 
-            $_ = '';
-        }
-        elsif( /^\%UNDEFINE(?:_(SCRIPT|VARIABLE))?\s+([^][\s]+)/mso ) {
-            # Undefining a macro, script or variable
-            my $which = $1 || 'MACRO';
+		$_ = '';
+	}
+	elsif( /^\%DEFINE(?:_(SCRIPT|VARIABLE))?\s+([^][\s]+)\s*\[(.*?)\]/mso ) {
+		# Defining a single-line macro, script or variable
+		my $which = $1 || 'MACRO';
 
-            carp "Cannot undefine non-existent $which $2 $where" 
-            unless $self->_remove_element( $which, $2 ); 
-     
-            $_ = '';
-        }
-        elsif( /^\%UNDEFINE_ALL(?:_(SCRIPT|VARIABLE))?/mso ) {
-            # Undefining all macros or scripts
-            my $which = "-".lc($1 || 'MACRO');
-			$self->undefine_all($which);
+		$self->_insert_element( $which, $2, $self->_expand_variable( $3 || '' ) );
 
-            $_ = '';
-        }
-        elsif( /^\%DEFINE(?:_(SCRIPT|VARIABLE))?\s+([^][\s]+)\s*\[(.*?)\]/mso ) {
-            # Defining a single-line macro, script or variable
-            my $which = $1 || 'MACRO';
+		$_ = '';
+	}
+	elsif( /^\%DEFINE(?:_(SCRIPT))?\s+([^][\s]+)/mso ) {
+		# Preparing to define a multi-line macro or script (we don't permit
+		# multi-line variables)
+		$self->cur_name($2);
+		$self->cur_define('');
+		if (defined $1) {
+			$self->in_script(1);
+		}
+		else {
+			$self->in_macro(1);
+		}
 
-            $self->_insert_element( $which, $2, $self->_expand_variable( $3 || '' ) );
+		$_ = '';
+	}
+	elsif( /^\%(LOAD|INCLUDE)\s*\[(.+?)\]/mso ) {
+		# Save state in local stack frame (i.e. recursion is taking care of
+		# stacking for us)
+		my $in_macro    = $self->in_macro;   # Should never be true
+		my $in_script   = $self->in_script;  # Should never be true
+		my $in_case     = $self->in_case;    # Should never be true
+		my $in_embedded = $self->in_embedded;
+		my $name        = $self->cur_name;
+		my $define      = $self->cur_define;
+		my $line_nr     = $self->line_nr;
 
-            $_ = '';
-        }
-        elsif( /^\%DEFINE(?:_(SCRIPT))?\s+([^][\s]+)/mso ) {
-            # Preparing to define a multi-line macro or script (we don't permit
-            # multi-line variables)
-            $self->cur_name($2);
-            $self->cur_define('');
-			if (defined $1) {
-				$self->in_script(1);
-			}
-			else {
-				$self->in_macro(1);
-			}
+		my @lines = ();
+		
+		# Load in new stuff
+		if( $1 eq 'LOAD' ) {
+			# If we are doing embedded processing and we are loading a new
+			# file then we assume we are still in the embedded text since
+			# we're loading macros, scripts etc.
+			carp "Should be embedded when LOADing $2" 
+			if ($self->embedded && ! $self->in_embedded);
 
-            $_ = '';
-        }
-        elsif( /^\%(LOAD|INCLUDE)\s*\[(.+?)\]/mso ) {
-            # Save state in local stack frame (i.e. recursion is taking care of
-            # stacking for us)
-            my $in_macro    = $self->in_macro;   # Should never be true
-            my $in_script   = $self->in_script;  # Should never be true
-            my $in_case     = $self->in_case;    # Should never be true
-            my $in_embedded = $self->in_embedded;
-            my $name        = $self->cur_name;
-            my $define      = $self->cur_define;
-            my $line_nr     = $self->line_nr;
+			# $self->in_embedded(1); Should be 1 anyway - this is done
 
-            my @lines = ();
-            
-            # Load in new stuff
-            if( $1 eq 'LOAD' ) {
-                # If we are doing embedded processing and we are loading a new
-                # file then we assume we are still in the embedded text since
-                # we're loading macros, scripts etc.
-                carp "Should be embedded when LOADing $2" 
-                if ($self->embedded && ! $self->in_embedded);
+			# inside load_file().
+			# This is a macro/scripts file; instantiates macros and scripts,
+			# ignores everything else.
+			$self->load_file( $2 );
+		}
+		else {
+			# If we are doing embedded processing and we are including a new file
+			# then we assume that we are not in embedded text at the start of that
+			# file, i.e. we look freshly for an opening delimiter. 
+			carp "Should be embedded when INCLUDINGing $2" 
+			if ($self->embedded && ! $self->in_embedded);
 
-                # $self->in_embedded(1); Should be 1 anyway - this is done
+			$self->in_embedded(0); # Should be 1, but we want it off now. 
 
-                # inside load_file().
-                # This is a macro/scripts file; instantiates macros and scripts,
-                # ignores everything else.
-                $self->load_file( $2 );
-            }
-            else {
-                # If we are doing embedded processing and we are including a new file
-                # then we assume that we are not in embedded text at the start of that
-                # file, i.e. we look freshly for an opening delimiter. 
-                carp "Should be embedded when INCLUDINGing $2" 
-                if ($self->embedded && ! $self->in_embedded);
+			# This is a normal file that may contain macros/scripts - the
+			# macros and scripts are instantiated and any text is returned
+			# with all expansions applied
+			@lines = $self->expand_file( $2 );
+		}
+	
+		# Restore state
+		$self->in_macro($in_macro);
+		$self->in_script($in_script);
+		$self->in_case($in_case);
+		$self->in_embedded($in_embedded);
+		$self->cur_name($name);
+		$self->cur_define($define);
+		$self->line_nr($line_nr);
 
-                $self->in_embedded(0); # Should be 1, but we want it off now. 
+		# Replace string with the outcome of the load (empty) or include 
+		$_ = join '', @lines;
+	}
+	elsif( /^\%REQUIRE\s*\[(.+?)\]/mso ) {
+		my $file = $1;
+		eval {
+			require $file;
+		};
+		carp "Failed to require $file: $@" if $@;
 
-                # This is a normal file that may contain macros/scripts - the
-                # macros and scripts are instantiated and any text is returned
-                # with all expansions applied
-                @lines = $self->expand_file( $2 );
-            }
-        
-            # Restore state
-            $self->in_macro($in_macro);
-            $self->in_script($in_script);
-            $self->in_case($in_case);
-            $self->in_embedded($in_embedded);
-            $self->cur_name($name);
-            $self->cur_define($define);
-            $self->line_nr($line_nr);
+		$_ = '';
+	}
+	else {
+		# This array is already ordered by length then by ASCII.
+		foreach my $script ( @{$self->SCRIPT} ) {
+			my( $name, $orig_body ) = @{$script};
+			# We substitute wherever found, including in the middle of 'words'
+			# or whatever (but we can always create macro names like *MYMACRO
+			# which are unlikely to occur in words). 
+			# Macro names shouldn't include ] and can't include [.
+			s{
+				\Q$name\E
+				(?:\[(.+?)\])?  
+			 }{
+				# Get any parameters
+				my @param = split /\|/, $1 if defined $1;
+				# We get $body fresh every time since we could have the same
+				# macro or script occur more than once in a line but of course
+				# with different parameters.
+				my $body  = $orig_body;
+				# Substitute any parameters in the script's body; we go from
+				# largest index to smallest to ensure that we substitute #13
+				# before #1!
+				if( $body =~ /#\d/mso ) {
 
-            # Replace string with the outcome of the load (empty) or include 
-            $_ = join '', @lines;
-        }
-        elsif( /^\%REQUIRE\s*\[(.+?)\]/mso ) {
-            my $file = $1;
-            eval {
-                require $file;
-            };
-            carp "Failed to require `$file': $@" if $@;
+					$body =~ s/\\#/\x0/msgo; # Hide escaped #s
 
-            $_ = '';
-        }
-        else {
-            # This array is already ordered by length then by ASCII.
-            foreach my $script ( @{$self->SCRIPT} ) {
-                my( $name, $orig_body ) = @{$script};
-                # We substitute wherever found, including in the middle of 'words'
-                # or whatever (but we can always create macro names like *MYMACRO
-                # which are unlikely to occur in words). 
-                # Macro names shouldn't include ] and can't include [.
-                s{
-                    \Q$name\E
-                    (?:\[(.+?)\])?  
-                 }{
-                    # Get any parameters
-                    my @param = split /\|/, $1 if defined $1;
-                    # We get $body fresh every time since we could have the same
-                    # macro or script occur more than once in a line but of course
-                    # with different parameters.
-                    my $body  = $orig_body;
-                    # Substitute any parameters in the script's body; we go from
-                    # largest index to smallest to ensure that we substitute #13
-                    # before #1!
-                    if( $body =~ /#\d/mso ) {
+					# Warnings don't seem to work correctly here so we switch
+					# them off and do them manually.
+					local $^W = 0;
 
-                        $body =~ s/\\#/\x0/msgo; # Hide escaped #s
+					for( my $i = $#param; $i >= 0; $i-- ) {
+						$body =~ s/#$i/$param[$i]/msg;
+					}
+					carp "Missing parameter or unescaped # in SCRIPT " .
+						 "$name $body $where"
+					if ( $#param > 9 and $body =~ /#\d\d\D/mso ) or 
+									   ( $body =~ /#\d\D/mso );
 
-                        # Warnings don't seem to work correctly here so we switch
-                        # them off and do them manually.
-                        local $^W = 0;
+					$body =~ s/\x0/#/msgo; # Unhide escaped #s
+					# Extra parameters, i.e. those given in the text but not
+					# used by the macro or script are ignored and do not
+					# appear in the output.
+				}
+				# Evaluate the script 
+				my $result = '';
+				eval {
+					my @Param = @param; # Give (local)  access to params
+					my %Var   = %{$self->VARIABLE};
+					local $_;
+					$result   = eval $body;
+					%{$self->VARIABLE} = %Var;
+				};
+				croak "Evaluation of SCRIPT $name failed $where: $@" 
+				if $@;
+				# This carp does't work - its supposed to catch a failed eval
+				# and give an error message - instead perl doesn't set $@ but
+				# outputs its own error message immediately instead. Although
+				# we can switch off perl's message using local $^W = 0, doing
+				# so means that the error goes by silently, so I've left the
+				# default behaviour so at least we know we've got an error.
+				# Please let me know how to fix this!
 
-                        for( my $i = $#param; $i >= 0; $i-- ) {
-                            $body =~ s/#$i/$param[$i]/msg;
-                        }
-                        carp "missing parameter or unescaped # in SCRIPT " .
-                             "$name $body $where"
-                        if ( $#param > 9 and $body =~ /#\d\d\D/mso ) or 
-                                           ( $body =~ /#\d\D/mso );
+				# Return the result of the evaluation as the replacement string
+				$result;
+			 }gmsex; 
+		}
 
-                        $body =~ s/\x0/#/msgo; # Unhide escaped #s
-                        # Extra parameters, i.e. those given in the text but not
-                        # used by the macro or script are ignored and do not
-                        # appear in the output.
-                    }
-                    # Evaluate the script 
-                    my $result = '';
-                    eval {
-                        my @Param = @param; # Give (local)  access to params
-                        my %Var   = %{$self->VARIABLE};
-                        local $_;
-                        $result   = eval $body;
-                        %{$self->VARIABLE} = %Var;
-                    };
-                    croak "evaluation of SCRIPT $name failed $where: $@" 
-                    if $@;
-                    # This carp does't work - its supposed to catch a failed eval
-                    # and give an error message - instead perl doesn't set $@ but
-                    # outputs its own error message immediately instead. Although
-                    # we can switch off perl's message using local $^W = 0, doing
-                    # so means that the error goes by silently, so I've left the
-                    # default behaviour so at least we know we've got an error.
-                    # Please let me know how to fix this!
+		foreach my $macro ( @{$self->MACRO} ) {
+			my( $name, $body ) = @{$macro};
 
-                    # Return the result of the evaluation as the replacement string
-                    $result;
-                 }gmsex; 
-            }
+			s{
+				\Q$name\E
+				(?:\[(.+?)\])?  
+			 }{
+				my @param = split /\|/, $1 if defined $1;
+				{
+					$body =~ s/\\#/\x0/msgo; # Hide escaped #s
 
-            foreach my $macro ( @{$self->MACRO} ) {
-                my( $name, $body ) = @{$macro};
+					local $^W = 0;
 
-                s{
-                    \Q$name\E
-                    (?:\[(.+?)\])?  
-                 }{
-                    my @param = split /\|/, $1 if defined $1;
-                    {
-                        $body =~ s/\\#/\x0/msgo; # Hide escaped #s
+					for( my $i = $#param; $i >= 0; $i-- ) {
+						$body =~ s/#$i/$param[$i]/msg;
+					}
 
-                        local $^W = 0;
+					carp "Missing parameter or unescaped # in MACRO " .
+						 "$name $body $where"
+					if ( $#param > 9 and $body =~ /#\d\d\D/mso ) or 
+									   ( $body =~ /#\d\D/mso );
 
-                        for( my $i = $#param; $i >= 0; $i-- ) {
-                            $body =~ s/#$i/$param[$i]/msg;
-                        }
-
-                        carp "missing parameter or unescaped # in MACRO " .
-                             "$name $body $where"
-                        if ( $#param > 9 and $body =~ /#\d\d\D/mso ) or 
-                                           ( $body =~ /#\d\D/mso );
-
-                        $body =~ s/\x0/#/msgo; # Unhide escaped #s
-                    }
-                    $body;
-                 }gmsex; 
-            }
-        }
-    };
-    croak $@ if $@;
+					$body =~ s/\x0/#/msgo; # Unhide escaped #s
+				}
+				$body;
+			 }gmsex; 
+		}
+	}
 
     $_;
 }
@@ -825,10 +810,10 @@ Text::MacroScript - A macro pre-processor with embedded perl capability
         print $Macro->expand( $_ ) if $_;
     }
 
-    # Canonical use (the filename improves error messages):
+    # Canonical use (the filename and line number improves error messages):
     my $Macro = Text::MacroScript->new;
     while( <> ) {
-        print $Macro->expand( $_, $ARGV ) if $_;
+        print $Macro->expand( $_, $ARGV, $. ) if $_;
     }
 
     # new() for embedded macro processing
@@ -838,7 +823,7 @@ Text::MacroScript - A macro pre-processor with embedded perl capability
     # or
     my $Macro = Text::MacroScript->new( -opendelim => '[[', -closedelim => ']]' );
     while( <> ) {
-        print $Macro->expand_embedded( $_, $ARGV ) if $_;
+        print $Macro->expand_embedded( $_, $ARGV, $. ) if $_;
     }
 
     # Create a macro object and create initial macros/scripts from the file(s)
@@ -917,12 +902,12 @@ Text::MacroScript - A macro pre-processor with embedded perl capability
     # expand()
 
     $expanded = $Macro->expand( $unexpanded );
-    $expanded = $Macro->expand( $unexpanded, $filename );
+    $expanded = $Macro->expand( $unexpanded, $filename, $line_nr );
 
     # expand_embedded()
 
     $expanded = $Macro->expand_embedded( $unexpanded );
-    $expanded = $Macro->expand_embedded( $unexpanded, $filename );
+    $expanded = $Macro->expand_embedded( $unexpanded, $filename, $line_nr );
 
 
 This bundle also includes the C<macropp> and C<macrodir> scripts which allows us
@@ -977,8 +962,8 @@ delimiters of C<E<lt>:> and C<:E<gt>> throughout.
 
 =head3 new
 
-  $self = $class->new();
-  $self = $class->new( %opts );
+  $self = Text::MacroScript->new();
+  $self = Text::MacroScript->new( %opts );
 
 Create a new C<Text::MacroScript> object, initialized with the supplied 
 options. By default creates an object for macro processing. 
@@ -1259,14 +1244,15 @@ See also L</%UNDEFINE_ALL_VARIABLE>.
 =head3 expand
 
   $text = $Macro->expand( $in );
-  $text = $Macro->expand( $in, $filename );
+  $text = $Macro->expand( $in, $filename, $line_nr );
 
 Expands the given C<$in> input and returns the expanded text. The C<$in> 
 is either a text line or an interator that returns a sequence of text 
 lines. 
 
-The C<$filename> is optional and defaults to C<"-">. It is used in error 
-messages to locate the error. 
+The C<$filename> is optional and defaults to C<"-">. The <$line_nr> is
+optional and defaults to C<1>. They are used in error messages to locate 
+the error. 
 
 The expansion processes any macro definitions and expands any macro 
 calls found in the input text. C<expand()> buffers internally all the 
@@ -1277,7 +1263,7 @@ for each line of a multi-line L</%DEFINE>.
 =head3 expand_embedded
 
   $text = $Macro->expand_embedded( $in );
-  $text = $Macro->expand_embedded( $in, $filename );
+  $text = $Macro->expand_embedded( $in, $filename, $line_nr );
 
 Similar to C<expand()>, but only expands text between the open and close 
 delimiters (see C<-opendelim> and C<-closedelim>).
